import sys, time
import operator as op

VERBOSE=1

def dist(a,b,n): 
    """How far are positions a and b apart 
    on a line that wraps around of length n?"""
    d=abs(a-b)
    if d>n/2: 
        d=n-d
    return d

def distance(edges,pos,swap=None): 
    """Average distance for all edges for thing i at pos j"""

    if swap!=None:
        pos=list(pos)
        t=pos[swap[0]]
        pos[swap[0]]=pos[swap[1]]
        pos[swap[1]]=t

    d=0
    c=0
    for e in edges:
        a,b=pos[e[0]],pos[e[1]]
        w=1
        if len(e)>2: w=e[2]**3
        w+=0.01
        
        di=dist(a,b,len(pos))
        d+=w*di**2
        
    #d/=float(len(edges))
    return d

def improve(edges, pos, swap): 
    i=0
    for e in edges: 
        if e[0] in swap or e[1] in swap: 
            
            w=1
            if len(e)>2: w=e[2]
            w+=0.01            
            
            a,b=pos[e[0]],pos[e[1]]
            o=dist(a,b,len(pos))
            
            if e[0]==swap[0]:
                a=pos[swap[1]]
            elif e[0]==swap[1]:
                a=pos[swap[0]]

            if e[1]==swap[1]:
                b=pos[swap[0]]
            elif e[1]==swap[0]: 
                b=pos[swap[1]]

            n=dist(a,b,len(pos))
            i+=w*(n-o)
    return i

    

def optimise(nodes, edges, iterations=20, rand=0): 
    """edges are tuples (a,b)
    nodes are whatever
    return nodes in new order and new edges"""
    pos=range(len(nodes))
    if rand>0:
        import random
        scores=[]
        for i in range(rand):
            sys.stderr.write("Randomized run %d/%d\n"%(i,rand))
            random.shuffle(pos)
            n,e=_optimise(nodes,edges,iterations, pos)
            scores.append((i,distance(e,range(len(n))),n,e))
        scores.sort(key=op.itemgetter(1))
        sys.stderr.write("Worst run %d, score %.2f\n"%(scores[-1][0],scores[-1][1]))
        sys.stderr.write("Best run %d, score %.2f\n"%(scores[0][0],scores[0][1]))
        return scores[0][2:]
    else:
        return _optimise(nodes, edges, iterations, pos)
    
def _optimise(nodes, edges, iterations, pos):


    start=time.time()
    last=distance(edges,pos)
    sys.stderr.write("Average Distance %.2f\n"%last)
    for i in range(iterations):
        
        
        for a in range(len(nodes)):
        
            mind=9999999
            minb=-1
            #base=distance(edges,pos)
            for b in range(len(nodes)): 
                if a==b: continue
            
                #d=distance(edges,pos,(a,b))-base
                d=improve(edges, pos, (a,b))
                if VERBOSE>2: 
                    sys.stderr.write("Swapping %s-%s gives %.2f\n"%(nodes[a][0],nodes[b][0],d))
                if d<mind: 
                    mind=d
                    minb=b
            if minb==-1: continue
            if mind>=0: 
                if VERBOSE>1: 
                    sys.stderr.write("No improvement for %s, leaving\n"%nodes[a][0])
                continue
            if VERBOSE>1:
                sys.stderr.write("MinD: %s\n"%mind)
                sys.stderr.write("Swapping %s with %s\n"%(nodes[a][0],nodes[minb][0]))
            t=pos[a]
            pos[a]=pos[minb]
            pos[minb]=t

        
        sys.stderr.write("Iteration %d took %.2f seconds.\n"%(i, time.time()-start))
        start=time.time()

        this=distance(edges,pos)
        sys.stderr.write("Average Distance %.2f\n"%this)
        if this==last: 
            sys.stderr.write("No improvement, stopping.\n")
            break
        last=this
    

    resnodes=[None]*len(nodes)
    for x in range(len(pos)):
        resnodes[pos[x]]=nodes[x]
    resedges=[]
    for e in edges:
        resedges.append((pos[e[0]], pos[e[1]])+tuple(e[2:]))


    return resnodes, resedges
    
        

        
    
